# Generated by roxygen2: do not edit by hand

S3method("[",msm.est)
S3method(as.data.frame,msm.estbystate)
S3method(coef,msm)
S3method(contour,msm)
S3method(image,msm)
S3method(logLik,msm)
S3method(model.frame,msm)
S3method(model.matrix,msm)
S3method(persp,msm)
S3method(plot,msm)
S3method(plot,prevalence.msm)
S3method(print,hmmMVdist)
S3method(print,hmmdist)
S3method(print,hmodel)
S3method(print,msm)
S3method(print,msm.est)
S3method(print,msm.estbystate)
S3method(print,msm.prevalence)
S3method(print,pearson.msm)
S3method(print,summary.msm)
S3method(summary,msm)
S3method(tidy,msm)
S3method(tidy,msm.est)
S3method(tidy,msm.estbystate)
S3method(tidy,msm.prevalence)
export(MatrixExp)
export(absorbing.msm)
export(boot.msm)
export(coef.msm)
export(crudeinits.msm)
export(d2phase)
export(deltamethod)
export(dmenorm)
export(dmeunif)
export(dpexp)
export(draic.msm)
export(drlcv.msm)
export(dtnorm)
export(efpt.msm)
export(ematrix.msm)
export(envisits.msm)
export(h2phase)
export(hazard.msm)
export(hmmBeta)
export(hmmBetaBinom)
export(hmmBinom)
export(hmmCat)
export(hmmExp)
export(hmmGamma)
export(hmmIdent)
export(hmmLNorm)
export(hmmMETNorm)
export(hmmMEUnif)
export(hmmMV)
export(hmmNBinom)
export(hmmNorm)
export(hmmPois)
export(hmmT)
export(hmmTNorm)
export(hmmUnif)
export(hmmWeibull)
export(hmodel2list)
export(logLik.msm)
export(lrtest.msm)
export(msm)
export(msm.form.eoutput)
export(msm.form.qoutput)
export(msm2Surv)
export(odds.msm)
export(p2phase)
export(pearson.msm)
export(phasemeans.msm)
export(plot.prevalence.msm)
export(plot.survfit.msm)
export(plotprog.msm)
export(pmatrix.msm)
export(pmatrix.piecewise.msm)
export(pmenorm)
export(pmeunif)
export(pnext.msm)
export(ppass.msm)
export(ppexp)
export(prevalence.msm)
export(printnew.msm)
export(printold.msm)
export(ptnorm)
export(q2phase)
export(qgeneric)
export(qmatrix.msm)
export(qmenorm)
export(qmeunif)
export(qpexp)
export(qratio.msm)
export(qtnorm)
export(r2phase)
export(rmenorm)
export(rmeunif)
export(rpexp)
export(rtnorm)
export(scoreresid.msm)
export(sim.msm)
export(simfitted.msm)
export(simmulti.msm)
export(sojourn.msm)
export(statetable.msm)
export(surface.msm)
export(totlos.msm)
export(transient.msm)
export(updatepars.msm)
export(viterbi.msm)
importFrom(expm,expm)
importFrom(generics,tidy)
importFrom(grDevices,rainbow)
importFrom(graphics,contour)
importFrom(graphics,filled.contour)
importFrom(graphics,image)
importFrom(graphics,legend)
importFrom(graphics,lines)
importFrom(graphics,par)
importFrom(graphics,persp)
importFrom(graphics,plot)
importFrom(graphics,text)
importFrom(mvtnorm,rmvnorm)
importFrom(stats,as.formula)
importFrom(stats,coef)
importFrom(stats,deriv)
importFrom(stats,dexp)
importFrom(stats,dnorm)
importFrom(stats,integrate)
importFrom(stats,logLik)
importFrom(stats,model.extract)
importFrom(stats,model.frame)
importFrom(stats,model.matrix)
importFrom(stats,na.fail)
importFrom(stats,na.omit)
importFrom(stats,na.pass)
importFrom(stats,numericDeriv)
importFrom(stats,optimHess)
importFrom(stats,pchisq)
importFrom(stats,pexp)
importFrom(stats,plogis)
importFrom(stats,pnorm)
importFrom(stats,qlogis)
importFrom(stats,qnorm)
importFrom(stats,quantile)
importFrom(stats,rbeta)
importFrom(stats,rbinom)
importFrom(stats,reformulate)
importFrom(stats,reshape)
importFrom(stats,rexp)
importFrom(stats,rgamma)
importFrom(stats,rlnorm)
importFrom(stats,rnbinom)
importFrom(stats,rnorm)
importFrom(stats,rpois)
importFrom(stats,rt)
importFrom(stats,runif)
importFrom(stats,rweibull)
importFrom(stats,sd)
importFrom(stats,setNames)
importFrom(stats,terms)
importFrom(stats,uniroot)
importFrom(survival,Surv)
importFrom(survival,survfit)
importFrom(tibble,tibble)
importFrom(utils,head)
importFrom(utils,tail)
useDynLib(msm, .registration=TRUE)
